#!/usr/bin/env python3
"""CLI wrapper for running QMK commands.

This program can be run from anywhere, with or without a qmk_firmware checkout. It provides a small set of subcommands for working with QMK and otherwise dispatches to the `qmk_firmware/bin/qmk` script for the repo you are currently in, or your default repo if you are not currently in a qmk_firmware checkout.
"""
import argparse
import os
import subprocess
import sys
from glob import glob
from time import strftime
from functools import lru_cache
from importlib import import_module
from importlib.util import find_spec
from pathlib import Path


@lru_cache(maxsize=2)
def in_qmk_firmware():
    """Returns the path to the qmk_firmware we are currently in, or None if we are not inside qmk_firmware.
    """
    cur_dir = Path.cwd()
    while len(cur_dir.parents) > 0:
        found_bin = cur_dir / 'bin' / 'qmk'
        if found_bin.is_file():
            command = [found_bin, '--version']
            result = subprocess.run(command)

            if result.returncode == 0:
                return cur_dir

        # Move up a directory before the next iteration
        cur_dir = cur_dir / '..'
        cur_dir = cur_dir.resolve()


def find_qmk_firmware():
    """Look for qmk_firmware in the usual places.

    This function returns the path to qmk_firmware, or the default location if one does not exist.

    FIXME(skullydazed): add config file support
    """
    if in_qmk_firmware():
        return in_qmk_firmware()

    if 'QMK_HOME' in os.environ:
        return Path(os.environ['QMK_HOME'])

    return Path.home() / 'qmk_firmware'


def parse_args():
    """Process arguments outside milc.
    """
    parser = argparse.ArgumentParser(description='CLI wrapper for running QMK commands.')
    parser.add_argument('-H', '--home', help='Path to the qmk_firmware directory.')
    parser.add_argument('subcommand', help='Subcommand to run')
    parser.add_argument('subcommand_args', nargs=argparse.REMAINDER, help='Arguments to pass to the subcommand.')
    args = parser.parse_args()

    if args.home:
        os.environ['QMK_HOME'] = args.home

    return (args.subcommand, args.subcommand_args)


def main():
    subcommand, subcommand_args = parse_args()
    subcommand_module = 'qmk_cli.' + subcommand
    sys.argv = ['qmk-'+subcommand] + subcommand_args[1:]
    qmk_firmware = find_qmk_firmware()
    qmk_bin = qmk_firmware / 'bin' / 'qmk'
    os.environ['QMK_HOME'] = str(qmk_firmware)

    try:
        # Attempt to import the subcommand from qmk_cli first
        import qmk_cli.milc
        import_module(subcommand_module)
        qmk_cli.milc.cli()

    except ModuleNotFoundError as e:
        # Check to make sure there's not a bad import statement in qmk_cli
        if e.name != subcommand_module:
            raise

        # Dispatch to the underlying `qmk_firmware/bin/qmk`
        if qmk_bin.is_file() and os.access(str(qmk_bin), os.X_OK):
            argv = ['python3', str(qmk_bin), subcommand] + subcommand_args
            os.execvp('python3', argv)

        # Tell the user we can't continue
        print('*** Could not locate qmk_firmware directory!')
        exit(255)


if __name__ == '__main__':
    main()



"""QMK Python Doctor

Check up for QMK environment.
"""
import shutil
import platform
import os
from pathlib import Path

from qmk_cli.milc import cli


def check_qmk_firmware():
    """Make sure qmk_firmware and the qmk cli are there.
    """
    qmk_firmware = Path(os.environ['QMK_HOME'])
    qmk_cli = qmk_firmware / 'bin' / 'qmk'

    if qmk_firmware.exists() and not qmk_cli.exists():
        cli.log.error("{fg_red}Can't find %s/bin/qmk! You need to `git pull`.", os.environ['QMK_HOME'])
        return False

    elif not qmk_firmware.exists():
        cli.log.error("{fg_red}Can't find the qmk_firmware checkout! %s does not exist!", os.environ['QMK_HOME'])
        return False

    cli.log.info('Found qmk_firmware checkout in {fg_cyan}%s', str(qmk_firmware))
    return True


def check_vital_programs():
    """Make sure the software we need has been installed.

    TODO(unclaimed):
        * [ ] Run the binaries to make sure they work
        * [ ] Compile a trivial program with each compiler
    """
    ok = True
    binaries = ['dfu-programmer', 'avrdude', 'dfu-util', 'avr-gcc', 'arm-none-eabi-gcc']

    for binary in binaries:
        res = shutil.which(binary)
        if res is None:
            cli.log.error("{fg_red}QMK can't find %s in your path", binary)
            ok = False

    if ok:
        cli.log.info("All necessary software is installed.")

    return ok


def check_platform_tests():
    """Dispatch to platform specific tests.
    """
    OS = platform.system()

    if OS == "Darwin":
        cli.log.info("Detected {fg_cyan}macOS")
        return check_mac_os()

    elif OS == "Linux":
        cli.log.info("Detected {fg_cyan}linux")
        return check_linux()

    else:
        cli.log.info("Assuming {fg_cyan}Windows")
        return check_windows()


def check_mac_os():
    """Run macOS specific tests.

    There aren't any yet.
    """
    return True


def check_linux():
    """Run Linux specific tests.

    TODO(unclaimed):
        * [ ] Check for udev entries on linux
    """
    test = 'systemctl list-unit-files | grep enabled | grep -i ModemManager'
    if os.system(test) == 0:
        cli.log.warn("{bg_yellow}Detected modem manager. Please disable it if you are using Pro Micros")


def check_windows():
    """Run Windows specific tests.

    TODO(unclaimed):
        * [ ] Check out the driver situation
    """
    return True


@cli.entrypoint('Basic QMK environment checks')
def main(cli):
    """Basic QMK environment checks.

    This is currently very simple, it just checks that all the expected binaries are on your system.
    """
    cli.log.info('QMK Doctor is checking your environment')

    funcs = (
        check_qmk_firmware,
        check_vital_programs,
        check_platform_tests,
    )

    ok = True
    for func in funcs:
        if not func():
            ok = False

    if ok:
        cli.log.info('{fg_green}QMK is ready to go')



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['qmk_cli']

package_data = \
{'': ['*']}

install_requires = \
['argcomplete', 'colorama']

entry_points = \
{'console_scripts': ['qmk = qmk_cli.script_qmk:main']}

setup(name='qmk',
      version='0.0.1',
      description='A program to help you work with qmk_firmware.',
      author='skullydazed',
      author_email='skullydazed@gmail.com',
      url='https://github.com/qmk/qmk_cli',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.5',
     )
